From 4f8ddbe580ae71f3fc3435069afcfbc3d2a72634 Mon Sep 17 00:00:00 2001
From: Guo Xinle <guoxinle1@huawei.com>
Date: Tue, 23 Feb 2021 12:49:46 +0800
Subject: [PATCH 3/3] Block: fix process_queue func handle exception

If frond end set wrong format elem, after block handle
exception, it will never call add_used func to free the
descriptor table entry.

Signed-off-by: Guo Xinle <guoxinle1@huawei.com>
---
 device_model/src/virtio/block.rs | 28 +++++++++++++++++++++-------
 1 file changed, 21 insertions(+), 7 deletions(-)

diff --git a/device_model/src/virtio/block.rs b/device_model/src/virtio/block.rs
index e04ef26..21b60fb 100644
--- a/device_model/src/virtio/block.rs
+++ b/device_model/src/virtio/block.rs
@@ -452,11 +452,17 @@ impl BlockIoHandler {
                     req_index += 1;
                 }
                 Err(ref e) => {
+                    //  If it fails, also need to free descriptor table entry.
+                    queue
+                        .vring
+                        .add_used(&self.mem_space, elem.index, 0)
+                        .chain_err(|| "Failed to add used ring")?;
+                    need_interrupt = true;
+
                     error!(
                         "failed to create block request, {}",
                         error_chain::ChainedError::display_chain(e)
                     );
-                    break;
                 }
             };
         }
@@ -1280,13 +1286,17 @@ mod tests {
             .unwrap();
 
         signal_process_queue(mem_space.clone(), queue_config, queue_evt).unwrap();
-        // Block process queue failed, so no interrupt is injected.
-        assert!(interrupt_evt.read().is_err());
+
+        assert!(interrupt_evt.read().is_ok());
         // Get used_ring data
         let idx = mem_space
             .read_object::<u16>(GuestAddress(queue_config.used_ring.0 + 2 as u64))
             .unwrap();
-        assert_eq!(idx, 0);
+        let len = mem_space
+            .read_object::<u16>(GuestAddress(queue_config.used_ring.0 + 8 as u64))
+            .unwrap();
+        assert_eq!(idx, 1);
+        assert_eq!(len, 0);
     }
 
     // Set RequestOutHeader invalid type, block can not process request.
@@ -1332,13 +1342,17 @@ mod tests {
             .unwrap();
 
         signal_process_queue(mem_space.clone(), queue_config, queue_evt).unwrap();
-        // Block process queue failed, so no interrupt is injected.
-        assert!(interrupt_evt.read().is_err());
+
+        assert!(interrupt_evt.read().is_ok());
         // Get used_ring data
         let idx = mem_space
             .read_object::<u16>(GuestAddress(queue_config.used_ring.0 + 2 as u64))
             .unwrap();
-        assert_eq!(idx, 0);
+        let len = mem_space
+            .read_object::<u16>(GuestAddress(queue_config.used_ring.0 + 8 as u64))
+            .unwrap();
+        assert_eq!(idx, 1);
+        assert_eq!(len, 0);
     }
 
     // It will not work if the address set is out of bounds.
-- 
2.27.0

